Method: Hash#pluralized_array

Defined in:
lib/jekyll/core_ext.rb

#pluralized_array(singular_key, plural_key) ⇒ Object

Read array from the supplied hash favouring the singular key and then the plural key, and handling any nil entries.

+hash+ the hash to read from
+singular_key+ the singular key
+plural_key+ the plural key

Returns an array



30
31
32
33
34
35
36
37
38
39
40
41
42
43
# File 'lib/jekyll/core_ext.rb', line 30

def pluralized_array(singular_key, plural_key)
  hash = self
  if hash.has_key?(singular_key)
    array = [hash[singular_key]] if hash[singular_key]
  elsif hash.has_key?(plural_key)
    case hash[plural_key]
    when String
      array = hash[plural_key].split
    when Array
      array = hash[plural_key].compact
    end
  end
  array || []
end