Method: Jekyll::Collection#directory
- Defined in:
- lib/jekyll/collection.rb
#directory ⇒ Object
The full path to the directory containing the collection.
Returns a String containing th directory name where the collection
is stored on the filesystem.
117 118 119 120 121 |
# File 'lib/jekyll/collection.rb', line 117 def directory @directory ||= site.in_source_dir( File.join(container, relative_directory) ) end |