Class: Jekyll::Page

Inherits:
Object
  • Object
show all
Includes:
Convertible
Defined in:
lib/jekyll/page.rb

Direct Known Subclasses

PageWithoutAFile

Constant Summary collapse

ATTRIBUTES_FOR_LIQUID =

Attributes for Liquid templates

%w(
  content
  dir
  excerpt
  name
  path
  url
).freeze
HTML_EXTENSIONS =

A set of extensions that are considered HTML or HTML-like so we should not alter them, this includes .xhtml through XHTM5.

%w(
  .html
  .xhtml
  .htm
).freeze

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Convertible

#[], #asset_file?, #coffeescript_file?, #converters, #do_layout, #hook_owner, #invalid_layout?, #output_ext, #place_in_layout?, #published?, #read_yaml, #render_all_layouts, #render_liquid, #render_with_liquid?, #renderer, #sass_file?, #to_liquid, #to_s, #transform, #type, #validate_data!, #validate_permalink!, #write

Constructor Details

#initialize(site, base, dir, name) ⇒ Page

Initialize a new Page.

site - The Site object. base - The String path to the source. dir - The String path between the source and the file. name - The String filename of the file.



39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
# File 'lib/jekyll/page.rb', line 39

def initialize(site, base, dir, name)
  @site = site
  @base = base
  @dir  = dir
  @name = name
  @path = if site.in_theme_dir(base) == base # we're in a theme
            site.in_theme_dir(base, dir, name)
          else
            site.in_source_dir(base, dir, name)
          end

  process(name)
  read_yaml(PathManager.join(base, dir), name)
  generate_excerpt if site.config["page_excerpts"]

  data.default_proc = proc do |_, key|
    site.frontmatter_defaults.find(relative_path, type, key)
  end

  Jekyll::Hooks.trigger :pages, :post_init, self
end

Instance Attribute Details

#basenameObject

Returns the value of attribute basename.



9
10
11
# File 'lib/jekyll/page.rb', line 9

def basename
  @basename
end

#contentObject

Returns the value of attribute content.



10
11
12
# File 'lib/jekyll/page.rb', line 10

def content
  @content
end

#dataObject

Returns the value of attribute data.



10
11
12
# File 'lib/jekyll/page.rb', line 10

def data
  @data
end

#dirObject

The generated directory into which the page will be placed upon generation. This is derived from the permalink or, if permalink is absent, will be '/'

Returns the String destination directory.



66
67
68
# File 'lib/jekyll/page.rb', line 66

def dir
  url.end_with?("/") ? url : url_dir
end

#extObject Also known as: extname

Returns the value of attribute ext.



9
10
11
# File 'lib/jekyll/page.rb', line 9

def ext
  @ext
end

#nameObject

Returns the value of attribute name.



9
10
11
# File 'lib/jekyll/page.rb', line 9

def name
  @name
end

#outputObject

Returns the value of attribute output.



10
11
12
# File 'lib/jekyll/page.rb', line 10

def output
  @output
end

#pagerObject

Returns the value of attribute pager.



8
9
10
# File 'lib/jekyll/page.rb', line 8

def pager
  @pager
end

#siteObject

Returns the value of attribute site.



8
9
10
# File 'lib/jekyll/page.rb', line 8

def site
  @site
end

Instance Method Details

#destination(dest) ⇒ Object

Obtain destination path.

dest - The String path to the destination dir.

Returns the destination file path String.



155
156
157
158
159
160
161
162
163
# File 'lib/jekyll/page.rb', line 155

def destination(dest)
  @destination ||= {}
  @destination[dest] ||= begin
    path = site.in_dest_dir(dest, URL.unescape_path(url))
    path = File.join(path, "index") if url.end_with?("/")
    path << output_ext unless path.end_with? output_ext
    path
  end
end

#excerptObject



192
193
194
195
196
# File 'lib/jekyll/page.rb', line 192

def excerpt
  return @excerpt if defined?(@excerpt)

  @excerpt = data["excerpt"] ? data["excerpt"].to_s : nil
end

#excerpt_separatorObject



188
189
190
# File 'lib/jekyll/page.rb', line 188

def excerpt_separator
  @excerpt_separator ||= (data["excerpt_separator"] || site.config["excerpt_separator"]).to_s
end

#generate_excerpt?Boolean

Returns:

  • (Boolean)


198
199
200
# File 'lib/jekyll/page.rb', line 198

def generate_excerpt?
  !excerpt_separator.empty? && instance_of?(Jekyll::Page) && html?
end

#html?Boolean

Returns the Boolean of whether this Page is HTML or not.

Returns:

  • (Boolean)


171
172
173
# File 'lib/jekyll/page.rb', line 171

def html?
  HTML_EXTENSIONS.include?(output_ext)
end

#index?Boolean

Returns the Boolean of whether this Page is an index file or not.

Returns:

  • (Boolean)


176
177
178
# File 'lib/jekyll/page.rb', line 176

def index?
  basename == "index"
end

#inspectObject

Returns the object as a debug String.



166
167
168
# File 'lib/jekyll/page.rb', line 166

def inspect
  "#<#{self.class} @relative_path=#{relative_path.inspect}>"
end

#pathObject

The path to the source file

Returns the path to the source file



141
142
143
# File 'lib/jekyll/page.rb', line 141

def path
  data.fetch("path") { relative_path }
end

The full path and filename of the post. Defined in the YAML of the post body.

Returns the String permalink or nil if none has been set.



74
75
76
# File 'lib/jekyll/page.rb', line 74

def permalink
  data.nil? ? nil : data["permalink"]
end

#process(name) ⇒ Object

Extract information from the page filename.

name - The String filename of the page file.

NOTE: `String#gsub` removes all trailing periods (in comparison to `String#chomp`) Returns nothing.



118
119
120
121
122
123
# File 'lib/jekyll/page.rb', line 118

def process(name)
  return unless name

  self.ext = File.extname(name)
  self.basename = name[0..-ext.length - 1].gsub(%r!\.*\z!, "")
end

#relative_pathObject

The path to the page source file, relative to the site source



146
147
148
# File 'lib/jekyll/page.rb', line 146

def relative_path
  @relative_path ||= PathManager.join(@dir, @name).sub(%r!\A/!, "")
end

#render(layouts, site_payload) ⇒ Object

Add any necessary layouts to this post

layouts - The Hash of => “layout”. site_payload - The site payload Hash.

Returns String rendered page.



131
132
133
134
135
136
# File 'lib/jekyll/page.rb', line 131

def render(layouts, site_payload)
  site_payload["page"] = to_liquid
  site_payload["paginator"] = pager.to_liquid

  do_layout(site_payload, layouts)
end

#templateObject

The template of the permalink.

Returns the template String.



81
82
83
84
85
86
87
88
89
# File 'lib/jekyll/page.rb', line 81

def template
  if !html?
    "/:path/:basename:output_ext"
  elsif index?
    "/:path/"
  else
    Utils.add_permalink_suffix("/:path/:basename", site.permalink_style)
  end
end

#trigger_hooks(hook_name, *args) ⇒ Object



180
181
182
# File 'lib/jekyll/page.rb', line 180

def trigger_hooks(hook_name, *args)
  Jekyll::Hooks.trigger :pages, hook_name, self, *args
end

#urlObject

The generated relative url of this page. e.g. /about.html.

Returns the String url.



94
95
96
97
98
99
100
# File 'lib/jekyll/page.rb', line 94

def url
  @url ||= URL.new(
    :template     => template,
    :placeholders => url_placeholders,
    :permalink    => permalink
  ).to_s
end

#url_placeholdersObject

Returns a hash of URL placeholder names (as symbols) mapping to the desired placeholder replacements. For details see “url.rb”



104
105
106
107
108
109
110
# File 'lib/jekyll/page.rb', line 104

def url_placeholders
  {
    :path       => @dir,
    :basename   => basename,
    :output_ext => output_ext,
  }
end

#write?Boolean

Returns:

  • (Boolean)


184
185
186
# File 'lib/jekyll/page.rb', line 184

def write?
  true
end