Method: Jekyll::Collection#collection_dir
- Defined in:
- lib/jekyll/collection.rb
#collection_dir(*files) ⇒ Object
The full path to the directory containing the collection, with
optional subpaths.
*files - (optional) any other path pieces relative to the
directory to append to the path
Returns a String containing th directory name where the collection
is stored on the filesystem.
131 132 133 134 135 |
# File 'lib/jekyll/collection.rb', line 131 def collection_dir(*files) return directory if files.empty? site.in_source_dir(container, relative_directory, *files) end |