Method: Jekyll::Utils::Ansi#strip
- Defined in:
- lib/jekyll/utils/ansi.rb
#strip(str) ⇒ Object
Strip ANSI from the current string. It also strips cursor stuff, well some of it, and it also strips some other stuff that a lot of the other ANSI strippers don’t.
25 26 27 |
# File 'lib/jekyll/utils/ansi.rb', line 25 def strip(str) str.gsub MATCH, "" end |